import fuzzylite as fl


class Discrete:
    def __init__(self) -> None:
        self.engine = fl.Engine(
            name="Discrete",
            description="obstacle avoidance for self-driving cars",
            input_variables=[
                fl.InputVariable(
                    name="obstacle",
                    description="location of obstacle relative to vehicle",
                    minimum=0.0,
                    maximum=1.0,
                    lock_range=False,
                    terms=[
                        fl.Triangle("left", 0.0, 0.333, 0.666),
                        fl.Triangle("right", 0.333, 0.666, 1.0),
                    ],
                )
            ],
            output_variables=[
                fl.OutputVariable(
                    name="steer",
                    description="direction to steer the vehicle to",
                    minimum=0.0,
                    maximum=1.0,
                    lock_range=False,
                    lock_previous=False,
                    default_value=fl.nan,
                    aggregation=fl.Maximum(),
                    defuzzifier=fl.Centroid(),
                    terms=[
                        fl.Discrete(
                            "left",
                            fl.array(
                                [
                                    fl.array([0.0, 0.0]),
                                    fl.array([0.00666, 0.02]),
                                    fl.array([0.01332, 0.04]),
                                    fl.array([0.01998, 0.06]),
                                    fl.array([0.02664, 0.08]),
                                    fl.array([0.0333, 0.1]),
                                    fl.array([0.03996, 0.12]),
                                    fl.array([0.04662, 0.14]),
                                    fl.array([0.05328, 0.16]),
                                    fl.array([0.05994, 0.18]),
                                    fl.array([0.0666, 0.2]),
                                    fl.array([0.07326, 0.22]),
                                    fl.array([0.07992, 0.24]),
                                    fl.array([0.08658, 0.26]),
                                    fl.array([0.09324, 0.28]),
                                    fl.array([0.0999, 0.3]),
                                    fl.array([0.10656, 0.32]),
                                    fl.array([0.11322, 0.34]),
                                    fl.array([0.11988, 0.36]),
                                    fl.array([0.12654, 0.38]),
                                    fl.array([0.1332, 0.4]),
                                    fl.array([0.13986, 0.42]),
                                    fl.array([0.14652, 0.44]),
                                    fl.array([0.15318, 0.46]),
                                    fl.array([0.15984, 0.48]),
                                    fl.array([0.1665, 0.5]),
                                    fl.array([0.17316, 0.52]),
                                    fl.array([0.17982, 0.54]),
                                    fl.array([0.18648, 0.56]),
                                    fl.array([0.19314, 0.58]),
                                    fl.array([0.1998, 0.6]),
                                    fl.array([0.20646, 0.62]),
                                    fl.array([0.21312, 0.64]),
                                    fl.array([0.21978, 0.66]),
                                    fl.array([0.22644, 0.68]),
                                    fl.array([0.2331, 0.7]),
                                    fl.array([0.23976, 0.72]),
                                    fl.array([0.24642, 0.74]),
                                    fl.array([0.25308, 0.76]),
                                    fl.array([0.25974, 0.78]),
                                    fl.array([0.2664, 0.8]),
                                    fl.array([0.27306, 0.82]),
                                    fl.array([0.27972, 0.84]),
                                    fl.array([0.28638, 0.86]),
                                    fl.array([0.29304, 0.88]),
                                    fl.array([0.2997, 0.9]),
                                    fl.array([0.30636, 0.92]),
                                    fl.array([0.31302, 0.94]),
                                    fl.array([0.31968, 0.96]),
                                    fl.array([0.32634, 0.98]),
                                    fl.array([0.333, 1.0]),
                                    fl.array([0.33966, 0.98]),
                                    fl.array([0.34632, 0.96]),
                                    fl.array([0.35298, 0.94]),
                                    fl.array([0.35964, 0.92]),
                                    fl.array([0.3663, 0.9]),
                                    fl.array([0.37296, 0.88]),
                                    fl.array([0.37962, 0.86]),
                                    fl.array([0.38628, 0.84]),
                                    fl.array([0.39294, 0.82]),
                                    fl.array([0.3996, 0.8]),
                                    fl.array([0.40626, 0.78]),
                                    fl.array([0.41292, 0.76]),
                                    fl.array([0.41958, 0.74]),
                                    fl.array([0.42624, 0.72]),
                                    fl.array([0.4329, 0.7]),
                                    fl.array([0.43956, 0.68]),
                                    fl.array([0.44622, 0.66]),
                                    fl.array([0.45288, 0.64]),
                                    fl.array([0.45954, 0.62]),
                                    fl.array([0.4662, 0.6]),
                                    fl.array([0.47286, 0.58]),
                                    fl.array([0.47952, 0.56]),
                                    fl.array([0.48618, 0.54]),
                                    fl.array([0.49284, 0.52]),
                                    fl.array([0.4995, 0.5]),
                                    fl.array([0.50616, 0.48]),
                                    fl.array([0.51282, 0.46]),
                                    fl.array([0.51948, 0.44]),
                                    fl.array([0.52614, 0.42]),
                                    fl.array([0.5328, 0.4]),
                                    fl.array([0.53946, 0.38]),
                                    fl.array([0.54612, 0.36]),
                                    fl.array([0.55278, 0.34]),
                                    fl.array([0.55944, 0.32]),
                                    fl.array([0.5661, 0.3]),
                                    fl.array([0.57276, 0.28]),
                                    fl.array([0.57942, 0.26]),
                                    fl.array([0.58608, 0.24]),
                                    fl.array([0.59274, 0.22]),
                                    fl.array([0.5994, 0.2]),
                                    fl.array([0.60606, 0.18]),
                                    fl.array([0.61272, 0.16]),
                                    fl.array([0.61938, 0.14]),
                                    fl.array([0.62604, 0.12]),
                                    fl.array([0.6327, 0.1]),
                                    fl.array([0.63936, 0.08]),
                                    fl.array([0.64602, 0.06]),
                                    fl.array([0.65268, 0.04]),
                                    fl.array([0.65934, 0.02]),
                                    fl.array([0.666, 0.0]),
                                ]
                            ),
                        ),
                        fl.Discrete(
                            "right",
                            fl.array(
                                [
                                    fl.array([0.333, 0.0]),
                                    fl.array([0.33967, 0.02003003]),
                                    fl.array([0.34634, 0.04006006]),
                                    fl.array([0.35301, 0.06009009]),
                                    fl.array([0.35968, 0.08012012]),
                                    fl.array([0.36635, 0.10015015]),
                                    fl.array([0.37302, 0.12018018]),
                                    fl.array([0.37969, 0.14021021]),
                                    fl.array([0.38636, 0.16024024]),
                                    fl.array([0.39303, 0.18027027]),
                                    fl.array([0.3997, 0.2003003]),
                                    fl.array([0.40637, 0.22033033]),
                                    fl.array([0.41304, 0.24036036]),
                                    fl.array([0.41971, 0.26039039]),
                                    fl.array([0.42638, 0.28042042]),
                                    fl.array([0.43305, 0.30045045]),
                                    fl.array([0.43972, 0.32048048]),
                                    fl.array([0.44639, 0.340510511]),
                                    fl.array([0.45306, 0.360540541]),
                                    fl.array([0.45973, 0.380570571]),
                                    fl.array([0.4664, 0.400600601]),
                                    fl.array([0.47307, 0.420630631]),
                                    fl.array([0.47974, 0.440660661]),
                                    fl.array([0.48641, 0.460690691]),
                                    fl.array([0.49308, 0.480720721]),
                                    fl.array([0.49975, 0.500750751]),
                                    fl.array([0.50642, 0.520780781]),
                                    fl.array([0.51309, 0.540810811]),
                                    fl.array([0.51976, 0.560840841]),
                                    fl.array([0.52643, 0.580870871]),
                                    fl.array([0.5331, 0.600900901]),
                                    fl.array([0.53977, 0.620930931]),
                                    fl.array([0.54644, 0.640960961]),
                                    fl.array([0.55311, 0.660990991]),
                                    fl.array([0.55978, 0.681021021]),
                                    fl.array([0.56645, 0.701051051]),
                                    fl.array([0.57312, 0.721081081]),
                                    fl.array([0.57979, 0.741111111]),
                                    fl.array([0.58646, 0.761141141]),
                                    fl.array([0.59313, 0.781171171]),
                                    fl.array([0.5998, 0.801201201]),
                                    fl.array([0.60647, 0.821231231]),
                                    fl.array([0.61314, 0.841261261]),
                                    fl.array([0.61981, 0.861291291]),
                                    fl.array([0.62648, 0.881321321]),
                                    fl.array([0.63315, 0.901351351]),
                                    fl.array([0.63982, 0.921381381]),
                                    fl.array([0.64649, 0.941411411]),
                                    fl.array([0.65316, 0.961441441]),
                                    fl.array([0.65983, 0.981471471]),
                                    fl.array([0.6665, 0.998502994]),
                                    fl.array([0.67317, 0.978532934]),
                                    fl.array([0.67984, 0.958562874]),
                                    fl.array([0.68651, 0.938592814]),
                                    fl.array([0.69318, 0.918622754]),
                                    fl.array([0.69985, 0.898652695]),
                                    fl.array([0.70652, 0.878682635]),
                                    fl.array([0.71319, 0.858712575]),
                                    fl.array([0.71986, 0.838742515]),
                                    fl.array([0.72653, 0.818772455]),
                                    fl.array([0.7332, 0.798802395]),
                                    fl.array([0.73987, 0.778832335]),
                                    fl.array([0.74654, 0.758862275]),
                                    fl.array([0.75321, 0.738892216]),
                                    fl.array([0.75988, 0.718922156]),
                                    fl.array([0.76655, 0.698952096]),
                                    fl.array([0.77322, 0.678982036]),
                                    fl.array([0.77989, 0.659011976]),
                                    fl.array([0.78656, 0.639041916]),
                                    fl.array([0.79323, 0.619071856]),
                                    fl.array([0.7999, 0.599101796]),
                                    fl.array([0.80657, 0.579131737]),
                                    fl.array([0.81324, 0.559161677]),
                                    fl.array([0.81991, 0.539191617]),
                                    fl.array([0.82658, 0.519221557]),
                                    fl.array([0.83325, 0.499251497]),
                                    fl.array([0.83992, 0.479281437]),
                                    fl.array([0.84659, 0.459311377]),
                                    fl.array([0.85326, 0.439341317]),
                                    fl.array([0.85993, 0.419371257]),
                                    fl.array([0.8666, 0.399401198]),
                                    fl.array([0.87327, 0.379431138]),
                                    fl.array([0.87994, 0.359461078]),
                                    fl.array([0.88661, 0.339491018]),
                                    fl.array([0.89328, 0.319520958]),
                                    fl.array([0.89995, 0.299550898]),
                                    fl.array([0.90662, 0.279580838]),
                                    fl.array([0.91329, 0.259610778]),
                                    fl.array([0.91996, 0.239640719]),
                                    fl.array([0.92663, 0.219670659]),
                                    fl.array([0.9333, 0.199700599]),
                                    fl.array([0.93997, 0.179730539]),
                                    fl.array([0.94664, 0.159760479]),
                                    fl.array([0.95331, 0.139790419]),
                                    fl.array([0.95998, 0.119820359]),
                                    fl.array([0.96665, 0.099850299]),
                                    fl.array([0.97332, 0.07988024]),
                                    fl.array([0.97999, 0.05991018]),
                                    fl.array([0.98666, 0.03994012]),
                                    fl.array([0.99333, 0.01997006]),
                                    fl.array([1.0, 0.0]),
                                ]
                            ),
                        ),
                    ],
                )
            ],
            rule_blocks=[
                fl.RuleBlock(
                    name="steer_away",
                    description="steer away from obstacles",
                    conjunction=None,
                    disjunction=None,
                    implication=fl.Minimum(),
                    activation=fl.General(),
                    rules=[
                        fl.Rule.create("if obstacle is left then steer is right"),
                        fl.Rule.create("if obstacle is right then steer is left"),
                    ],
                )
            ],
        )
